context("Decadal monthly data - 3")

###########################################

source("modules/Loading/Loading_decadal.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")

recipe_file <- "tests/recipes/recipe-decadal_monthly_3.yml"
recipe <- prepare_outputs(recipe_file)
archive <- read_yaml(paste0(recipe$Run$code_dir, "conf/archive_decadal.yml"))$archive

# Load datasets
suppressWarnings({invisible(capture.output(
data <- load_datasets(recipe)
))})

# Calibrate datasets
suppressWarnings({invisible(capture.output(
  calibrated_data <- calibrate_datasets(recipe, data)
))})

# Compute skill metrics
suppressWarnings({invisible(capture.output(
skill_metrics <- compute_skill_metrics(recipe, calibrated_data)
))})
suppressWarnings({invisible(capture.output(
probs <- compute_probabilities(recipe, calibrated_data)
))})

#======================================

test_that("1. Loading", {

expect_equal(
is.list(data),
TRUE
)
expect_equal(
names(data),
c("hcst", "fcst", "obs")
)
expect_equal(
class(data$hcst),
"s2dv_cube"
)
expect_equal(
data$fcst,
NULL
)
expect_equal(
class(data$obs),
"s2dv_cube"
)
expect_equal(
names(data$hcst),
c("data", "lon", "lat", "Variable", "Datasets", "Dates", "when", "source_files", "load_parameters")
)
expect_equal(
names(data$hcst),
names(data$obs)
)
expect_equal(
dim(data$hcst$data),
c(dat = 1, var = 1, sday = 1, sweek = 1, syear = 4, time = 3, latitude = 25, longitude = 16, ensemble = 3)
)
expect_equal(
dim(data$hcst$Dates$start),
c(sday = 1, sweek = 1, syear = 4, time = 3)
)
# hcst data
expect_equal(
as.vector(aperm(drop(data$hcst$data), c(5, 1:4))[3, , 2, 2, 2]),
c(278.4305, 279.5065, 280.4110, 278.7608),
tolerance = 0.0001
)
expect_equal(
mean(data$hcst$data),
284.3765,
tolerance = 0.0001
)
expect_equal(
range(data$hcst$data),
c(263.3929, 300.4329),
tolerance = 0.0001
)

expect_equal(
(data$hcst$Dates$start)[1],
as.POSIXct("2016-04-16", tz = 'UTC')
)
expect_equal(
(data$hcst$Dates$start)[2],
as.POSIXct("2017-04-16", tz = 'UTC')
)
expect_equal(
(data$hcst$Dates$start)[5],
as.POSIXct("2016-05-16 12:00:00", tz = 'UTC')
)
expect_equal(
(data$hcst$Dates$start)[12],
as.POSIXct("2019-06-16", tz = 'UTC')
)

})

#======================================
test_that("2. Calibration", {

expect_equal(
names(calibrated_data),
c("hcst", "obs", "fcst")
)
expect_equal(
as.vector(aperm(drop(calibrated_data$hcst$data), c(5, 1:4))[3, , 2, 2, 2]),
c(279.0648, 281.0578, 282.6535, 280.3137),
tolerance = 0.0001
)
expect_equal(
calibrated_data$fcst,
NULL
)
})


##======================================
test_that("3. Metrics", {

expect_equal(
is.list(skill_metrics),
TRUE
)
expect_equal(
names(skill_metrics),
c("bss10", "bss10_significance", "corr", "corr_significance")
)
expect_equal(
class(skill_metrics[[1]]),
"array"
)
expect_equal(
all(unlist(lapply(lapply(skill_metrics, dim)[1:2], all.equal, c(time = 3, latitude = 25, longitude = 16)))),
TRUE
)
expect_equal(
all(unlist(lapply(lapply(skill_metrics, dim)[3:4], all.equal, c(ensemble = 3, time = 3, latitude = 25, longitude = 16)))),
TRUE
)
expect_equal(
as.vector(skill_metrics$bss10[, 1, 2]),
c(-0.1904762, -0.1904762, -0.1904762),
tolerance = 0.0001
)
expect_equal(
any(as.vector(skill_metrics$bss10_significance)),
FALSE
)
expect_equal(
as.vector(skill_metrics$corr[2, , 1, 2]),
c(-0.2015265, 0.4635463, -0.1019575),
tolerance = 0.0001
)

# Probs
expect_equal(
names(probs),
c('probs', 'percentiles')
)
expect_equal(
names(probs$probs),
c('prob_b33', 'prob_33_to_66', 'prob_a66')
)
expect_equal(
names(probs$percentiles),
c('percentile_33', 'percentile_66')
)
expect_equal(
dim(probs$probs$prob_b33),
c(syear = 4, time = 3, latitude = 25, longitude = 16)
)
expect_equal(
dim(probs$percentiles$percentile_33),
c(time = 3, latitude = 25, longitude = 16)
)
expect_equal(
as.vector(probs$probs$prob_b33[, 1, 2, 2]),
c(0.0, 0.3333333, 0.3333333, 0.6666667),
tolerance = 0.0001
)
expect_equal(
as.vector(probs$percentiles$percentile_33[1:3, 1, 2]),
c(278.1501, 279.5226, 282.0237),
tolerance = 0.0001
)

})


